"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.build = undefined;

var _bluebirdLst;

function _load_bluebirdLst() {
    return _bluebirdLst = require("bluebird-lst");
}

var _bluebirdLst2;

function _load_bluebirdLst2() {
    return _bluebirdLst2 = _interopRequireDefault(require("bluebird-lst"));
}

let build = exports.build = (() => {
    var _ref = (0, (_bluebirdLst || _load_bluebirdLst()).coroutine)(function* (rawOptions) {
        const options = normalizeOptions(rawOptions || {});
        if (options.cscLink === undefined && !(0, (_electronBuilderUtil || _load_electronBuilderUtil()).isEmptyOrSpaces)(process.env.CSC_LINK)) {
            options.cscLink = process.env.CSC_LINK;
        }
        if (options.cscInstallerLink === undefined && !(0, (_electronBuilderUtil || _load_electronBuilderUtil()).isEmptyOrSpaces)(process.env.CSC_INSTALLER_LINK)) {
            options.cscInstallerLink = process.env.CSC_INSTALLER_LINK;
        }
        if (options.cscKeyPassword === undefined && !(0, (_electronBuilderUtil || _load_electronBuilderUtil()).isEmptyOrSpaces)(process.env.CSC_KEY_PASSWORD)) {
            options.cscKeyPassword = process.env.CSC_KEY_PASSWORD;
        }
        if (options.cscInstallerKeyPassword === undefined && !(0, (_electronBuilderUtil || _load_electronBuilderUtil()).isEmptyOrSpaces)(process.env.CSC_INSTALLER_KEY_PASSWORD)) {
            options.cscInstallerKeyPassword = process.env.CSC_INSTALLER_KEY_PASSWORD;
        }
        if (options.draft === undefined && !(0, (_electronBuilderUtil || _load_electronBuilderUtil()).isEmptyOrSpaces)(process.env.EP_DRAFT)) {
            options.draft = process.env.EP_DRAFT.toLowerCase() === "true";
        }
        if (options.prerelease === undefined && !(0, (_electronBuilderUtil || _load_electronBuilderUtil()).isEmptyOrSpaces)(process.env.EP_PRELEASE)) {
            options.prerelease = process.env.EP_PRELEASE.toLowerCase() === "true";
        }
        const cancellationToken = new (_CancellationToken || _load_CancellationToken()).CancellationToken();
        const packager = new (_packager || _load_packager()).Packager(options, cancellationToken);
        // because artifact event maybe dispatched several times for different publish providers
        const artifactPaths = new Set();
        packager.artifactCreated(function (event) {
            if (event.file != null) {
                artifactPaths.add(event.file);
            }
        });
        const publishManager = new (_PublishManager || _load_PublishManager()).PublishManager(packager, options, cancellationToken);
        const buildPromise = packager.build().then(function () {
            return Array.from(artifactPaths);
        });
        process.on("SIGINT", function () {
            (0, (_log || _load_log()).warn)("Cancelled by SIGINT");
            cancellationToken.cancel();
            publishManager.cancelTasks();
        });
        return yield (0, (_promise || _load_promise()).executeFinally)(buildPromise, function (errorOccurred) {
            if (errorOccurred) {
                publishManager.cancelTasks();
                return (_bluebirdLst2 || _load_bluebirdLst2()).default.resolve(null);
            } else {
                return publishManager.awaitTasks();
            }
        });
    });

    return function build(_x) {
        return _ref.apply(this, arguments);
    };
})();
//# sourceMappingURL=builder.js.map


exports.normalizeOptions = normalizeOptions;
exports.createTargets = createTargets;

var _electronBuilderCore;

function _load_electronBuilderCore() {
    return _electronBuilderCore = require("electron-builder-core");
}

var _CancellationToken;

function _load_CancellationToken() {
    return _CancellationToken = require("electron-builder-http/out/CancellationToken");
}

var _electronBuilderUtil;

function _load_electronBuilderUtil() {
    return _electronBuilderUtil = require("electron-builder-util");
}

var _log;

function _load_log() {
    return _log = require("electron-builder-util/out/log");
}

var _promise;

function _load_promise() {
    return _promise = require("electron-builder-util/out/promise");
}

var _packager;

function _load_packager() {
    return _packager = require("./packager");
}

var _PublishManager;

function _load_PublishManager() {
    return _PublishManager = require("./publish/PublishManager");
}

var _targetFactory;

function _load_targetFactory() {
    return _targetFactory = require("./targets/targetFactory");
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function addValue(map, key, value) {
    const list = map.get(key);
    if (list == null) {
        map.set(key, [value]);
    } else {
        list.push(value);
    }
}
function normalizeOptions(args) {
    if (args.targets != null) {
        return args;
    }
    let targets = new Map();
    function processTargets(platform, types) {
        if (args.platform != null) {
            throw new Error(`--platform cannot be used if --${platform.buildConfigurationKey} is passed`);
        }
        if (args.arch != null) {
            throw new Error(`--arch cannot be used if --${platform.buildConfigurationKey} is passed`);
        }
        function commonArch() {
            const result = Array();
            if (args.x64) {
                result.push((_electronBuilderCore || _load_electronBuilderCore()).Arch.x64);
            }
            if (args.armv7l) {
                result.push((_electronBuilderCore || _load_electronBuilderCore()).Arch.armv7l);
            }
            if (args.ia32) {
                result.push((_electronBuilderCore || _load_electronBuilderCore()).Arch.ia32);
            }
            return result.length === 0 ? [(0, (_electronBuilderCore || _load_electronBuilderCore()).archFromString)(process.arch)] : result;
        }
        let archToType = targets.get(platform);
        if (archToType == null) {
            archToType = new Map();
            targets.set(platform, archToType);
        }
        if (types.length === 0) {
            const defaultTargetValue = args.dir ? [(_targetFactory || _load_targetFactory()).DIR_TARGET] : [];
            if (platform === (_electronBuilderCore || _load_electronBuilderCore()).Platform.MAC) {
                archToType.set((_electronBuilderCore || _load_electronBuilderCore()).Arch.x64, defaultTargetValue);
            } else {
                for (const arch of commonArch()) {
                    archToType.set(arch, defaultTargetValue);
                }
            }
            return;
        }
        for (const type of types) {
            let arch;
            if (platform === (_electronBuilderCore || _load_electronBuilderCore()).Platform.MAC) {
                arch = "x64";
                addValue(archToType, (_electronBuilderCore || _load_electronBuilderCore()).Arch.x64, type);
            } else {
                const suffixPos = type.lastIndexOf(":");
                if (suffixPos > 0) {
                    addValue(archToType, (0, (_electronBuilderCore || _load_electronBuilderCore()).archFromString)(type.substring(suffixPos + 1)), type.substring(0, suffixPos));
                } else {
                    for (const arch of commonArch()) {
                        addValue(archToType, arch, type);
                    }
                }
            }
        }
    }
    if (args.mac != null) {
        processTargets((_electronBuilderCore || _load_electronBuilderCore()).Platform.MAC, args.mac);
    }
    if (args.linux != null) {
        processTargets((_electronBuilderCore || _load_electronBuilderCore()).Platform.LINUX, args.linux);
    }
    if (args.win != null) {
        processTargets((_electronBuilderCore || _load_electronBuilderCore()).Platform.WINDOWS, args.win);
    }
    if (targets.size === 0) {
        if (args.platform == null && args.arch == null) {
            processTargets((_electronBuilderCore || _load_electronBuilderCore()).Platform.current(), []);
        } else {
            targets = createTargets((0, (_packager || _load_packager()).normalizePlatforms)(args.platform), args.dir ? (_targetFactory || _load_targetFactory()).DIR_TARGET : null, args.arch);
        }
    }
    const result = Object.assign({}, args);
    result.targets = targets;
    delete result.dir;
    delete result.mac;
    delete result.linux;
    delete result.win;
    delete result.platform;
    delete result.arch;
    const r = result;
    delete r.em;
    delete r.m;
    delete r.o;
    delete r.l;
    delete r.w;
    delete r.windows;
    delete r.macos;
    delete r.$0;
    delete r._;
    delete r.version;
    delete r.help;
    delete result.ia32;
    delete result.x64;
    delete result.armv7l;
    if (result.project != null && result.projectDir == null) {
        result.projectDir = result.project;
    }
    delete result.project;
    return result;
}
function createTargets(platforms, type, arch) {
    const targets = new Map();
    for (const platform of platforms) {
        const archs = platform === (_electronBuilderCore || _load_electronBuilderCore()).Platform.MAC ? [(_electronBuilderCore || _load_electronBuilderCore()).Arch.x64] : arch === "all" ? [(_electronBuilderCore || _load_electronBuilderCore()).Arch.x64, (_electronBuilderCore || _load_electronBuilderCore()).Arch.ia32] : [(0, (_electronBuilderCore || _load_electronBuilderCore()).archFromString)(arch == null ? process.arch : arch)];
        const archToType = new Map();
        targets.set(platform, archToType);
        for (const arch of archs) {
            archToType.set(arch, type == null ? [] : [type]);
        }
    }
    return targets;
}